/**
 * apeman app to verify email.
 * @memberof module:apeman-app-verify/lib
 * @function create
 * @param {object} UserVerify - User verify model.
 * @param {object} [options] - Optional settings.
 * @param {function} [options.beforeSend] - Handle before send
 * @param {function} [options.beforeVerify] - Handle before verify
 * @param {function} [options.afterSend] - Handle after send
 * @param {function} [options.afterVerify] - Handle after verify
 * @param {object} [sendSchema=defaultSchema('send')] - JSON schema for send.
 * @param {object} [verifySchema=defaultSchema('verify')] - JSON schema for verify.

 * @returns {function} - Defined app function.
 */

"use strict";

const argx = require('argx'),
    async = require('async'),
    assert = require('assert'),
    route = require('apeman-app-route'),
    validate = require('apeman-app-validate'),
    noop = require('apeman-app-noop'),
    action = require('apeman-app-action'),
    doSend = require('./verifying/do_send'),
    doVerify = require('./verifying/do_verify'),
    getVerify = require('./verifying/get_verify'),
    schemas = require('./schemas'),
    objnest = require('objnest');

const debug = require('debug')('apeman:react:verify');

const ONE_DAY = 24 * 60 * 60 * 1000;

/** @lends create */
function create(UserVerify, options) {
    let args = argx(arguments);
    options = objnest.expand(args.pop('object') || {});

    let format = options.format;
    let expireDuration = options.expireDuration || ONE_DAY,
        beforeSend = options.beforeSend || noop(),
        beforeVerify = options.beforeVerify || noop(),
        afterSend = options.afterSend || noop(),
        afterVerify = options.afterVerify || noop();

    function sendResult(req, res) {
        let result = req.result;
        res.send(result, format);
    }

    let handlers = [
        (req, res, next) => {
            next();
        },
        route({
            '/send': {
                'POST': [
                    beforeSend,
                    validate(options.sendSchema || schemas('send')),
                    action('verify-send'),
                    function verifySend(req, res, next) {
                        let kind = req.body.kind,
                            signed = req.signed();
                        if (!signed) {
                            res.statusCode = 401;
                            res.send({msg: 'Needs signin'}, format);
                            return;
                        }

                        let user = signed.user;

                        function _send(kind, value) {
                            doSend(UserVerify, user.id, kind, value, expireDuration)
                                .then(verify => {
                                    let success = !!verify;
                                    if (!success) {
                                        res.statusCode = 400;
                                        res.send({msg: 'Verify not found.'}, format);
                                        return;
                                    }
                                    req.result = {success, verify};
                                    next();

                                    return verify;
                                });
                        }


                        switch (String(kind).toLowerCase()) {
                            case 'email':
                                _send(kind, user.email);
                                break;
                            default:
                                res.statusCode = 404;
                                res.send({msg: `Unknown kind: ${kind}`}, format);
                                break;
                        }
                    },
                    afterSend,
                    sendResult
                ]
            },
            '/verified': {
                'GET': [
                    action('verify-verified'),
                    function verifyVerified(req, res, next) {
                        let kind = req.query.kind,
                            signed = req.signed();
                        if (!signed) {
                            res.statusCode = 401;
                            res.send({
                                msg: "Needs sign-in"
                            }, format);
                            return;
                        }

                        let user = signed.user;

                        function _get(kind, value) {
                            getVerify(UserVerify, user.id, kind, value)
                                .then(verify => {
                                    req.result = {
                                        verified: !!(verify && verify.get('verified'))
                                    };
                                    next();
                                    return verify;
                                })
                                .catch(next);
                        }

                        switch (String(kind).toLowerCase()) {
                            case 'email':
                                _get(kind, user.email);
                                break;
                            default:
                                res.statusCode = 404;
                                res.send({msg: `Unknown kind: ${kind}`}, format);
                                break;
                        }
                    },
                    sendResult
                ]
            },
            '/verify': {
                'POST': [
                    beforeVerify,
                    validate(options.verifySchema || schemas('verify')),
                    action('verify-verify'),
                    function verifyVerify(req, res, next) {
                        let token = req.body.token;
                        doVerify(UserVerify, token)
                            .then(verify => {
                                if (!verify) {
                                    res.statusCode = 404;
                                    res.send({}, format);
                                    return verify;
                                }
                                let success = true;
                                req.result = {success, verify};
                                next();

                                return verify;
                            })
                            .catch(next)
                    },
                    afterVerify,
                    sendResult
                ]
            }
        })
    ];

    /**
     * Defined app.
     * @function app
     * @param {object} req - Request object.
     * @param {object} res - Response object.
     * @param {function} next - Pass to next handler.
     */
    function app(req, res, next) {
        assert(req.body, 'req.body not found. Use "apeman-app-form" before this.');
        assert(req.signed, 'req.signed not found. Use "apeman-app-signed" before this.');
        assert(res.header, 'req.header not found. Use "apeman-app-header" before this.');
        assert(res.send, 'req.send not found. Use "apeman-app-send" before this.');

        app.$handler(req, res, next);
    }

    app.$handler = function $handler(req, res, next) {
        async.eachSeries(handlers, (handler, next) => {
            handler(req, res, next);
        }, next);
    };

    app.$endpoints = handlers[1].$endpoints;


    // Description of this app.
    app.$desc = "Verify endpoint";
    return app;
}

module.exports = create;
