/**
 * Test case for create.
 * Runs with mocha.
 */
"use strict";

const create = require('../lib/create.js'),
    freeport = require('freeport'),
    apemanApp = require('apeman-app'),
    async = require('async'),
    fs = require('fs'),
    apemandb = require('apemandb'),
    request = require('request'),
    assert = require('assert');

let tmpDir = __dirname + '/../tmp',
    dbFile = tmpDir + '/testing-db01.db';

describe('create', () => {

    let db;
    let baseUrl;
    let close;
    let token01;
    let req = request.defaults({jar: true});

    before((done) => {
        db = apemandb({
            env: {
                DATABASE: 'testing-database02',
                USERNAME: 'FOO',
                PASSWORD: 'BAR',
                DIALECT: 'sqlite',
                STORAGE: dbFile,
                LOGGING: false
            },
            models: __dirname + '/../doc/mocks/models/*.json'
        });


        return db.sync({force: true})
            .then((db) => {
                done();
            });
    });

    before((done) => {
        let app = create(
            db.model('UserVerify'),
            {});
        assert.ok(app);
        freeport((err, port) => {
            assert.ifError(err);
            apemanApp('testing-app', {
                configuration: {
                    $apps: {
                        'testing-app': {
                            '/': [
                                require('apeman-app-json')(),
                                require('apeman-app-format')(),
                                require('apeman-app-query')(),
                                require('apeman-app-send')(),
                                require('apeman-app-header')(),
                                require('apeman-app-form')(),
                                require('apeman-app-session')(
                                    db.model('Session'), '1234qwer'
                                ),
                                require('apeman-app-signed')()
                            ],
                            '/api/sign': require('apeman-app-sign')(
                                db.model('UserSign')
                            ),
                            '/api/verify': app
                        }
                    }
                },
                port: port
            }, (err, server) => {
                assert.ifError(err);
                close = server.close.bind(server);
                baseUrl = 'http://localhost:' + port + '/';
                done(err);
            });
        });
    });

    after((done) => {
        close(()=> {
            setTimeout(() => {
                fs.unlink(dbFile, () => {
                    done();
                });
            }, 100); //Wait for flush.
        });
    });


    it('Send fail.', (done) => {
        req({
            url: baseUrl + 'api/verify/send',
            method: 'POST',
            form: {
                kind: '__invalid_kind___'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 400);
            done(err);
        });
    });

    it('Signup success', (done) => {
        req({
            url: baseUrl + 'api/sign/signup',
            method: 'POST',
            form: {
                username: 'user01',
                email: 'user01@exampole.com',
                password: 'user01_passwd'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.ok(data.success);
            assert.equal(data.signed[0].user_id, data.signed[1].user_id);
            done(err);
        })
    });

    it('Signin success', (done) => {
        req({
            url: baseUrl + 'api/sign/signin',
            method: 'POST',
            form: {
                sign_key: 'user01@exampole.com',
                password: 'user01_passwd'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.ok(data.success);
            done(err);
        })
    });

    it('Send success.', (done) => {
        req({
            url: baseUrl + 'api/verify/send',
            method: 'POST',
            form: {
                kind: 'email'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.ok(data.success);
            let verify = data.verify;
            token01 = verify.verify_token;
            assert.ok(token01);
            assert.equal(verify.verified, false);
            done(err);
        });
    });

    it('Check verified to false', (done) => {
        req({
            url: baseUrl + 'api/verify/verified?kind=email',
            method: 'GET'
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.equal(data.verified, false);
            done(err);
        });
    });

    it('Verify fail.', (done) => {
        req({
            url: baseUrl + 'api/verify/verify',
            method: 'POST',
            form: {
                token: '____invalid_token_____'
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 404);
            done(err);
        });
    });

    it('Verify success.', (done) => {
        req({
            url: baseUrl + 'api/verify/verify',
            method: 'POST',
            form: {
                token: token01
            }
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.equal(data.success, true);
            let verify = data.verify;
            assert.equal(verify.verified, true);
            done(err);
        });
    });

    it('Check verified to true', (done) => {
        req({
            url: baseUrl + 'api/verify/verified?kind=email',
            method: 'GET',
        }, (err, res, body) => {
            assert.equal(res.statusCode, 200);
            let data = JSON.parse(body);
            assert.equal(data.verified, true);
            done(err);
        });
    });
});

